# -*- coding: utf-8 -*-

"""Console script for heroku.env"""

# enable absolute imports of this module for Python2.x
from __future__ import absolute_import
from __future__ import unicode_literals  # unicode support for py2

import os

import click
import requests

from heroku_env import __version__
from .constants import (
    HEROKU_TROUBLESHOOT_URL,
    HEROKU_API_KEY_HELP_URL
)
from .exceptions import (
    HerokuRunError,
    InvalidHerokuAppError,
    InvalidAPIKeyError
)
from .heroku_env import upload_env
from .param_types import APIKeyParamType


@click.command(context_settings=dict(help_option_names=['-h', '--help']))
@click.version_option(__version__, '-v', '--version')
@click.option(
    '-a',
    '--app',
    required=True,
    type=str,
    help="The name of the heroku app."
)
@click.option(
    '-e',
    '--env-file',
    type=click.Path(exists=True, file_okay=True, dir_okay=False,
                    readable=True, resolve_path=True, allow_dash=False),
    default=".env",
    show_default=True,
    help="Path to the .env file"
)
@click.option(
    '-k',
    '--api-key',
    prompt="Please enter the Heroku API key to continue",
    hide_input=True,
    confirmation_prompt=True,
    required=True,
    type=APIKeyParamType(),
    envvar="HEROKU_API_KEY",
    help="Your Heroku API key"
)
@click.option(
    '-t',
    '--set-alt',
    is_flag=True,
    required=False,
    help="Flag to enable reading of alternate values of env vars from"
         " comments in the env file."
         " Specify the alternate value to use with 'alt_value=VALUE'"
         " in the line right before the actual env var you want to change."
)
def main(app, env_file, api_key, set_alt):
    """
    Simple CLI tool to upload environment variables to
    Heroku from a .env file, through the Heroku API.

    It is recommended for security purposes that you set API
    key as an environment variable like this:

    export HEROKU_API_KEY=a1b12c24-ab1d-123f-5678-1234b12a0a1b

    Example usages:

    heroku.env --app swimming-briskly-123 --env-file dot.env

    heroku.env --app swimming-briskly-123 --env-file dot.env
    --api-key a1b12c24-ab1d-123f-5678-1234b12a0a1b
    """
    # if not defined, then set it.
    if not os.getenv('HEROKU_API_KEY'):
        os.environ['HEROKU_API_KEY'] = api_key
    try:
        upload_env(app, env_file, set_alt)
    except IndexError:
        raise click.ClickException("The entries in your .env file are not of the form KEY=VALUE")
    except HerokuRunError as e:
        # launch Heroku troubleshooting page for a failed run.
        click.launch(HEROKU_TROUBLESHOOT_URL)
        raise click.ClickException(e)
    except InvalidAPIKeyError as e:
        # launch API key doc
        click.launch(HEROKU_API_KEY_HELP_URL)
        raise click.ClickException(e)
    except (
        HerokuRunError,
        InvalidHerokuAppError
    ) as e:
        raise click.ClickException(e)
    except requests.exceptions.ConnectionError:
        raise click.ClickException("Please check your internet connection and try again.")
    except Exception as e:
        # all other exceptions
        click.echo(e)
        raise click.ClickException("An unknown error occurred. Please open an issue with the log.")


if __name__ == "__main__":
    main()  # pragma: no cover



# -*- coding: utf-8 -*-

"""
heroku_env.exceptions
-----------------------
All exceptions used in the heroku.env code base are defined here.
"""


class HerokuEnvException(Exception):
    """
    Base exception. All other exceptions
    inherit from here.
    """


class HerokuRunError(HerokuEnvException):
    """
    Exception raised when the Heroku command fails.
    """


class InvalidHerokuAppError(HerokuEnvException):
    """
    Exception raised when the given Heroku app is invalid
    """


class InvalidAPIKeyError(HerokuEnvException):
    """
    Raised when the provided Heroku API key is wrong
    """



# -*- coding: utf-8 -*-

from __future__ import unicode_literals  # unicode support for py2

# Heroku links
HEROKU_TROUBLESHOOT_URL = "https://devcenter.heroku.com/categories/troubleshooting"
HEROKU_API_KEY_HELP_URL = "https://devcenter.heroku.com/articles/authentication#retrieving-the-api-token"



# -*- coding: utf-8 -*-

"""Module containing the core functionality."""

from __future__ import unicode_literals  # unicode support for py2

import os

import click
import heroku3
import requests

from .exceptions import (
    HerokuRunError,
    InvalidHerokuAppError,
    InvalidAPIKeyError
)


def update_config_vars(config_dict, app):
    """
    Use the Heroku API to update config vars

    :param config_dict: dictionary of config vars
    :param app: name of the Heroku app
    :return: None
    """
    updated_config = app.update_config(config_dict)
    return updated_config.to_dict()


def upload_env(app_name, env_file, set_alt):
    """
    Get and upload env vars to Heroku

    :param app_name: The Heroku app.
    :param env_file: Path to the env file.
    :param set_alt: Flag to check if alternate values must be used.
    :return: None
    """
    try:
        # key error isn't expected here since the CLI tool
        # forces an API key before it reaches this point.
        heroku_conn = heroku3.from_key(os.environ['HEROKU_API_KEY'])
        app_instance = heroku_conn.apps()[app_name]
    except requests.exceptions.HTTPError:
        raise InvalidAPIKeyError("Please check your API key and try again.")
    except KeyError:
        raise InvalidHerokuAppError(
            "We could not find a Heroku app named {} registered with your API key".format(app_name))

    # init
    config_dict = {}
    use_alt = False
    alt_value = ''

    with open(env_file) as e:

        for line in e:
            line = line.strip()
            if line:
                # check comments
                if line.startswith("#"):
                    # enable --set-alt
                    if set_alt and "alt_value=" in line:
                        alt_value = line.split("alt_value=", 1)[1]
                        if alt_value:
                            # use this value for the next env var
                            use_alt = True
                    # any kind of comment warrants a skip
                    continue

                # set env vars
                if "=" in line:
                    kv_pair = line.split("=", 1)
                    if len(kv_pair) > 1:  # has to at least be of form: k=
                        key = kv_pair[0]

                        if use_alt:
                            value = alt_value
                            # reset
                            use_alt = False
                        else:
                            value = kv_pair[1]

                        # an empty value is fine
                        if key:
                            config_dict[key] = value
                            # verify
                            click.secho(u'\u2713 ' + key, fg='green', bold=True)

                else:
                    click.echo("Skipping line : Not of the form key=value")

    # update
    update_result = update_config_vars(config_dict, app_instance)

    if not update_result:
        raise HerokuRunError("Failed to update env vars. Possibly an error with Heroku."
                             " Please contact Heroku support.")

    click.echo("Config vars updated successfully.")
